@@ -18,6 +18,7 @@ use tinymist_project::{LspComputeGraph, LspWorld, TaskWhen};
 use tinymist_std::hash::{FxDashMap, hash128};
 use tinymist_std::typst::TypstDocument;
 use tinymist_world::debug_loc::DataSource;
+use tinymist_world::package::registry::PackageIndexEntry;
 use tinymist_world::vfs::{PathResolution, WorkspaceResolver};
 use tinymist_world::{DETACHED_ENTRY, EntryReader};
 use typst::diag::{At, FileError, FileResult, SourceDiagnostic, SourceResult, StrResult};
@@ -25,7 +26,7 @@ use typst::foundations::{Bytes, IntoValue, Module, StyleChain, Styles};
 use typst::introspection::Introspector;
 use typst::layout::Position;
 use typst::model::BibliographyElem;
-use typst::syntax::package::{PackageManifest, PackageSpec};
+use typst::syntax::package::PackageManifest;
 use typst::syntax::{Span, VirtualPath};
 use typst_shim::eval::{Eval, eval_compat};
 
@@ -85,6 +86,8 @@ pub struct Analysis {
     pub periscope: Option<Arc<dyn PeriscopeProvider + Send + Sync>>,
     /// The global worker resources for analysis.
     pub workers: Arc<AnalysisGlobalWorkers>,
+    /// The local package cache.
+    pub local_packages: Arc<Mutex<OnceLock<EcoVec<PackageIndexEntry>>>>,
     /// The semantic token cache.
     pub tokens_caches: Arc<Mutex<SemanticTokenCache>>,
     /// The global caches for analysis.
@@ -169,6 +172,7 @@ impl Analysis {
         self.caches.def_signatures.clear();
         self.caches.static_signatures.clear();
         self.caches.terms.clear();
+        *self.local_packages.lock() = OnceLock::default();
         self.tokens_caches.lock().clear();
         self.analysis_rev_cache.lock().clear();
     }
@@ -324,8 +328,12 @@ impl DerefMut for LocalContext {
 impl LocalContext {
     /// Set list of packages for LSP-based completion.
     #[cfg(test)]
-    pub fn test_package_list(&mut self, f: impl FnOnce() -> Vec<(PackageSpec, Option<EcoString>)>) {
-        self.world().registry.test_package_list(f);
+    pub fn test_package_list(&mut self, f: impl FnOnce() -> Vec<PackageIndexEntry> + Clone) {
+        self.world().registry.test_package_list(f.clone());
+        self.analysis
+            .local_packages
+            .lock()
+            .get_or_init(|| f().into_iter().collect());
     }
 
     /// Set the files for LSP-based completion.
@@ -681,19 +689,19 @@ impl SharedContext {
         self.world().font_resolver.describe_font(&font)
     }
 
-    /// Gets the local packages and their descriptions.
-    #[cfg(feature = "local-registry")]
-    pub fn local_packages(&self) -> EcoVec<PackageSpec> {
-        crate::package::list_package_by_namespace(&self.world().registry, eco_format!("local"))
-            .into_iter()
-            .map(|(_, spec)| spec)
-            .collect()
-    }
-
-    /// Gets the local packages and their descriptions.
-    #[cfg(not(feature = "local-registry"))]
-    pub fn local_packages(&self) -> EcoVec<PackageSpec> {
-        eco_vec![]
+    /// Gets the packages other than that in the preview namespace and their
+    /// descriptions.
+    pub fn non_preview_packages(&self) -> EcoVec<PackageIndexEntry> {
+        #[cfg(feature = "local-registry")]
+        let it = || {
+            crate::package::list_package(
+                self.world(),
+                crate::package::PackageFilter::ExceptFor(EcoString::inline("preview")),
+            )
+        };
+        #[cfg(not(feature = "local-registry"))]
+        let it = || Default::default();
+        self.analysis.local_packages.lock().get_or_init(it).clone()
     }
 
     pub(crate) fn const_eval(rr: ast::Expr<'_>) -> Option<Value> {
