@@ -15,9 +15,11 @@ use tinymist_std::path::unix_slash;
 use tinymist_std::typst::TypstDocument;
 use tinymist_world::debug_loc::LspRange;
 use tinymist_world::package::PackageSpec;
+use tinymist_world::package::registry::PackageIndexEntry;
 use tinymist_world::vfs::WorkspaceResolver;
 use tinymist_world::{EntryReader, ShadowApi, TaskInputs};
 use typst::syntax::ast::{self, AstNode};
+use typst::syntax::package::PackageInfo;
 use typst::syntax::{LinkedNode, Source, SyntaxKind, VirtualPath};
 use typst_shim::syntax::LinkedNodeExt;
 
@@ -100,16 +102,41 @@ pub fn run_with_ctx_<T>(
     let mut ctx = a.enter_(g, a.lock_revision(None));
 
     ctx.test_package_list(|| {
-        vec![(
-            PackageSpec::from_str("@preview/example:0.1.0").unwrap(),
-            Some("example package (mock).".into()),
-        )]
+        vec![
+            dummy_package_from_spec(&PackageSpec::from_str("@preview/example:0.1.0").unwrap()),
+            dummy_package_from_spec(&PackageSpec::from_str("@preview/example:0.1.1").unwrap()),
+        ]
     });
     ctx.test_completion_files(|| paths.clone());
     ctx.test_files(|| paths);
     f(&mut ctx, path)
 }
 
+fn dummy_package_from_spec(spec: &PackageSpec) -> PackageIndexEntry {
+    PackageIndexEntry {
+        namespace: spec.namespace.clone(),
+        package: PackageInfo {
+            name: spec.name.clone(),
+            version: spec.version,
+            entrypoint: Default::default(),
+            authors: Default::default(),
+            license: Default::default(),
+            description: Some("example package (mock).".into()),
+            homepage: Default::default(),
+            repository: Default::default(),
+            keywords: Default::default(),
+            categories: Default::default(),
+            disciplines: Default::default(),
+            compiler: Default::default(),
+            exclude: Default::default(),
+            unknown_fields: Default::default(),
+        },
+        template: None,
+        updated_at: None,
+        path: None,
+    }
+}
+
 pub fn get_test_properties(s: &str) -> HashMap<&'_ str, &'_ str> {
     let mut props = HashMap::new();
     for line in s.lines() {
