@@ -393,45 +393,45 @@ impl LaTeXWriter {
                 match elem.tag.as_str() {
                     "thead" => {
                         for row_node in &elem.children {
-                            if let Node::HtmlElement(row) = row_node {
-                                if row.tag == "tr" {
-                                    let cells: Vec<Vec<Node>> = row
-                                        .children
-                                        .iter()
-                                        .filter_map(|cell_node| {
-                                            if let Node::HtmlElement(cell) = cell_node {
-                                                if cell.tag == "th" || cell.tag == "td" {
-                                                    return Some(cell.children.clone());
-                                                }
-                                            }
-                                            None
-                                        })
-                                        .collect();
-                                    col_count = col_count.max(cells.len());
-                                    headers.push(cells);
-                                }
+                            if let Node::HtmlElement(row) = row_node
+                                && row.tag == "tr"
+                            {
+                                let cells: Vec<Vec<Node>> = row
+                                    .children
+                                    .iter()
+                                    .filter_map(|cell_node| {
+                                        if let Node::HtmlElement(cell) = cell_node
+                                            && (cell.tag == "th" || cell.tag == "td")
+                                        {
+                                            return Some(cell.children.clone());
+                                        }
+                                        None
+                                    })
+                                    .collect();
+                                col_count = col_count.max(cells.len());
+                                headers.push(cells);
                             }
                         }
                     }
                     "tbody" => {
                         for row_node in &elem.children {
-                            if let Node::HtmlElement(row) = row_node {
-                                if row.tag == "tr" {
-                                    let cells: Vec<Vec<Node>> = row
-                                        .children
-                                        .iter()
-                                        .filter_map(|cell_node| {
-                                            if let Node::HtmlElement(cell) = cell_node {
-                                                if cell.tag == "th" || cell.tag == "td" {
-                                                    return Some(cell.children.clone());
-                                                }
-                                            }
-                                            None
-                                        })
-                                        .collect();
-                                    col_count = col_count.max(cells.len());
-                                    rows.push(cells);
-                                }
+                            if let Node::HtmlElement(row) = row_node
+                                && row.tag == "tr"
+                            {
+                                let cells: Vec<Vec<Node>> = row
+                                    .children
+                                    .iter()
+                                    .filter_map(|cell_node| {
+                                        if let Node::HtmlElement(cell) = cell_node
+                                            && (cell.tag == "th" || cell.tag == "td")
+                                        {
+                                            return Some(cell.children.clone());
+                                        }
+                                        None
+                                    })
+                                    .collect();
+                                col_count = col_count.max(cells.len());
+                                rows.push(cells);
                             }
                         }
                     }
@@ -441,10 +441,10 @@ impl LaTeXWriter {
                             .children
                             .iter()
                             .filter_map(|cell_node| {
-                                if let Node::HtmlElement(cell) = cell_node {
-                                    if cell.tag == "th" || cell.tag == "td" {
-                                        return Some(cell.children.clone());
-                                    }
+                                if let Node::HtmlElement(cell) = cell_node
+                                    && (cell.tag == "th" || cell.tag == "td")
+                                {
+                                    return Some(cell.children.clone());
                                 }
                                 None
                             })
