@@ -74,7 +74,7 @@ export async function wsMain({ url, previewMode, isContentPreview }: WsArgs) {
 
     if (!isContentPreview) {
       subsribes.push(
-        fromEvent(window, "scroll")
+        fromEvent(resizeTarget, "scroll")
           .pipe(debounceTime(500))
           .subscribe(() => svgDoc.addViewportChange()),
       );
@@ -183,11 +183,6 @@ export async function wsMain({ url, previewMode, isContentPreview }: WsArgs) {
     window.addEventListener("keydown", (e) => {
       let handled = true;
 
-      // https://stackoverflow.com/questions/3464876/javascript-get-window-x-y-position-for-scroll
-      let top = () => {
-        let doc = document.documentElement;
-        return (window.pageYOffset || doc.scrollTop) - (doc.clientTop || 0);
-      };
       const scrollDelta = 50;
 
       switch (e.key) {
@@ -209,16 +204,16 @@ export async function wsMain({ url, previewMode, isContentPreview }: WsArgs) {
           }
           break;
         case "j":
-          window.scroll({ top: top() + scrollDelta, behavior: "instant" });
+          resizeTarget.scrollBy({ top: + scrollDelta, behavior: "instant" });
           break;
         case "k":
-          window.scroll({ top: top() - scrollDelta, behavior: "instant" });
+          resizeTarget.scrollBy({ top: - scrollDelta, behavior: "instant" });
           break;
         case "h":
-          window.scroll({ top: top() - scrollDelta * 10, behavior: "smooth" });
+          resizeTarget.scrollBy({ top: - scrollDelta * 10, behavior: "smooth" });
           break;
         case "l":
-          window.scroll({ top: top() + scrollDelta * 10, behavior: "smooth" });
+          resizeTarget.scrollBy({ top: + scrollDelta * 10, behavior: "smooth" });
           break;
         case "?":
           blurInput();
